// ****************************************************************************
// *** LOGGING SYSTEM *********************************************************
// ****************************************************************************
var azureContext = null;
var setAzureContext = function(context) {
    azureContext = context;
};
var appName = "node";
var setAppName = function(name) {
    appName = name.toString().trim();
};
var getAppName = function() {
    return appName;
};
var logLevels = [ "TRACE", "DEBUG", "INFOR", "WARNG", "ERROR", "FATAL" ];
var logLevel = 2;
var setLevel = function(level) {
    if (typeof level === "number") {
        if (level>=0 && level<logLevels.length)
            logLevel = level;
    } else {
        level = level.toString().trim().toUpperCase();
        level = logLevels.indexOf(level);
        if (level>=0) logLevel = level;
    }
};
var getLevel = function() {
    return logLevels[logLevel];
};
var logMessageDate = function() {
    return (new Date()).toISOString().substring(0,23).replace("T"," ");
};
var logMessageApp = function() {
    return appName + ":" + process.pid + ":" + process.ppid;
}
var logMessage = function(message,name,level,multiLines) {
    if (level >= logLevel) {
        if (!multiLines && message != null)
            message = message.toString().trim().replace(/[\s]+/gm," ");
        var logMessage = "[" + logMessageDate() + "] <" + logMessageApp() + "> {" + name + "} (" + logLevels[level] + ") " + message;
        if (azureContext != null && azureContext.log != null) {
            try {
                var logAzureMessage = "(" + level + ") <" + logMessageApp() + "> {" + name + "} " + message;
                var logFunction = azureContext.log.info;
                if (level<2) logFunction = azureContext.log.verbose;
                else if (level == 3) logFunction = azureContext.log.warn;
                else if (level > 3) logFunction = azureContext.log.error;
                logFunction(logAzureMessage);
            } catch {
                console.log(logMessage);
            }
        } else console.log(logMessage);
    }
};
var newLog = function(name,multiLines) {
    return {
        t : function(message){return logMessage(message,name,0,multiLines);},
        d : function(message){return logMessage(message,name,1,multiLines);},
        i : function(message){return logMessage(message,name,2,multiLines);},
        w : function(message){return logMessage(message,name,3,multiLines);},
        e : function(message){return logMessage(message,name,4,multiLines);},
        f : function(message){return logMessage(message,name,5,multiLines);}
    };
};
// ****************************************************************************
var LOG = {
    setAzureContext : setAzureContext,
    setAppName      : setAppName,
    getAppName      : getAppName,
    setLevel        : setLevel,
    getLevel        : getLevel,
    newLog          : newLog
};
// ****************************************************************************
module.exports = LOG;